#include "stdafx.h"
#include "FilterWnd.h"
#include "resource.h"

CFilterWnd::CFilterWnd(HINSTANCE hInst)
{
	m_hInst = hInst;
	m_hWnd = NULL;
	m_hMenu = NULL;
	m_bPacketFilter = FALSE;
	m_lpSetFilterFunc = NULL;
	m_lpSetFilterContext = NULL;
	m_lpScheduleTimeFunc=NULL;
	m_lpScheduleTimeContext=NULL;
#ifdef FILTWND_ENABLE_TIPWND
	m_hTip=NULL;
#endif
	CLock::CreateLockObj(&m_stLock);
}

CFilterWnd::~CFilterWnd(void)
{
	while(m_Service.size()>0)
	{
		m_Service.begin()->second->pPIDs->clear();
		delete m_Service.begin()->second->pPIDs;
		delete m_Service.begin()->second;
		m_Service.erase(m_Service.begin()->first);
	}
	CLock::DestroyLockObj(&m_stLock);
}

BOOL CFilterWnd::Create(HINSTANCE hInst,int iResource,HWND hParentWnd,BOOL bNoPosSize,int x,int y,int sx,int sy,int iShowMode)
{
	m_hWnd=CreateDialogParam(hInst,MAKEINTRESOURCE(iResource),hParentWnd,(DLGPROC)_FilterDlg,(LPARAM)this);
	if(!bNoPosSize)
	{
		SetWindowPos(m_hWnd,0,x,y,sx,sy,SWP_NOZORDER);
	}
	ShowWindow(iShowMode);
	return TRUE;
}

void CFilterWnd::Destroy(int *pX ,int *pY,int *pSX,int *pSY,BOOL *pbShow)
{
	if(m_hWnd)
	{
		RECT rcRect;
		GetWindowRect(m_hWnd,&rcRect);
		if(pbShow) *pbShow=IsWindowVisible();
		if(pX) *pX=rcRect.left;
		if(pY) *pY=rcRect.top;
		if(pSX)*pSX=rcRect.right-rcRect.left;
		if(pSY)*pSY=rcRect.bottom-rcRect.top;
		if(m_hMenu) DestroyMenu(m_hMenu);
		DestroyWindow(m_hWnd);
	}
}

// tB^p_CAO
LRESULT CFilterWnd::FilterDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	CFilterWnd* pThis=this;

	switch (message)
	{
	case WM_INITDIALOG:
		{
			m_hWnd=hDlg;
			OnSize();
			SetEnable(FALSE);
			OnInitDialog();
			return TRUE;
		}
		break;
	case WM_DESTROY:
		{
#ifdef FILTWND_ENABLE_TIPWND
			if(m_hTip)
			{
				DestroyWindow(m_hTip);
				m_hTip=NULL;
			}
#endif
		}
		break;
	case WM_CLOSE:
		{
			ShowWindow(SW_HIDE);
		}
		break;

	case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
			case IDC_FILTER:
				{
					OnPacketFilterBtn();
				}
				break;
			case IDM_REC_SCHEDULE_CURRENT:
				{
					if(!OnRecSchedule(0))
					{
					}
				}
				break;
			case IDM_REC_SCHEDULE_NEXT:
				{
					if(!OnRecSchedule(1))
					{
					}
				}
				break;
			}
		}
		break;

	case WM_SIZE:
		{
			OnSize();
		}
		break;

	case WM_NOTIFY:
		{
			NMHDR *pNmHdr=(NMHDR*)lParam;
			if(pNmHdr->idFrom==IDC_SERVICE_LIST)
			{
				if(pNmHdr->code==NM_RCLICK)
				{
					// T[rXXgŉENbN
					int pos=ListView_GetNextItem(GetDlgItem(hDlg,IDC_SERVICE_LIST),-1,LVNI_ALL|LVNI_SELECTED);
					if(pos!=-1)
					{
						HMENU hPopup=GetSubMenu(m_hMenu,0);
						POINT pt;
						GetCursorPos(&pt);
						TrackPopupMenu(hPopup,TPM_LEFTALIGN,pt.x,pt.y,0,hDlg,NULL);
					}
				}
			}
		}
		break;
	
	case WM_FILTWND_STATUSRESET:
		{
			OnPacketFilterBtn(TRUE);
		}
		break;

	case WM_FILTWND_DELETEALL:
		{
			CLock lock(&m_stLock);
			ListView_DeleteAllItems(GetDlgItem(m_hWnd,IDC_SERVICE_LIST));
		}
		break;

	case WM_FILTWND_ADDITEM:
		{
			LV_ITEM *pstLI=(LV_ITEM*)lParam;
			if(pstLI)
			{
				CLock lock(&m_stLock);				
				HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
				if(LOWORD(wParam)==0)
				{
					ListView_SetItem(hListView,pstLI);
					if(pstLI->pszText) delete []pstLI->pszText;
				} else {
					FiltWndServiceListMap::iterator it = m_Service.find(LOWORD(wParam));
					if(it != m_Service.end())
					{
						ListView_InsertItem(hListView,pstLI);
						ListView_SetCheckState(hListView,pstLI->iItem,TRUE);
						// PMT
						wsprintf(pstLI->pszText,"0x%04X",HIWORD(wParam));
						PostMessage(m_hWnd,WM_FILTWND_SETSUBITEM,MAKEWPARAM(FILTWND_TAB_PMTPID,LOWORD(wParam)),(LPARAM)pstLI->pszText);
					}
#ifdef FILTWND_ENABLE_TIPWND
					UpdateToolTip(hListView);
#endif
				}
				delete pstLI;
			}
		}
		break;
	case WM_FILTWND_SETSUBITEM:
		{
			CLock lock(&m_stLock);
			HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
			WORD wServiceID=HIWORD(wParam);
			WORD wSubItemIndex=LOWORD(wParam);
			char *pszText=(char*)lParam;
			SetSubItem(wServiceID,wSubItemIndex,pszText);
#ifdef FILTWND_ENABLE_TIPWND
			if(pszText[0]) UpdateToolTip(hListView);
#endif
			delete []pszText;
		}
		break;

	case WM_FILTWND_NOTIFYSCROLL:
	case WM_FILTWND_NOTIFYMOUSEWHEEL:
		{
#ifdef FILTWND_ENABLE_TIPWND
			UpdateToolTip(GetDlgItem(hDlg,IDC_SERVICE_LIST));
#endif
		}
		break;
	}
	return FALSE;
}

// Subclass
static LRESULT CALLBACK ListViewExProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
#pragma warning(disable: 4312)
	WNDPROC lpOrigProc=reinterpret_cast<WNDPROC>(GetWindowLongPtr(hWnd,GWL_USERDATA));
#pragma warning(default: 4312)
	switch(message)
	{
	case WM_VSCROLL:
		{
			PostMessage(GetParent(hWnd),WM_FILTWND_NOTIFYSCROLL,0,0);
		}
		break;
	case 0x020A:// WM_MOUSEWHEEL:
		{
			PostMessage(GetParent(hWnd),WM_FILTWND_NOTIFYMOUSEWHEEL,0,0);
		}
		break;
	}
	return CallWindowProc(lpOrigProc,hWnd,message,wParam,lParam);
}

void CFilterWnd::OnInitDialog()
{	
	HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
	DWORD dwStyle=ListView_GetExtendedListViewStyle(hListView);
	dwStyle |= LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES | LVS_EX_CHECKBOXES;
	ListView_SetExtendedListViewStyle(hListView,dwStyle);

	// TuNX(bZ[Wǉʒmp)
	LONG_PTR LongPtr=GetWindowLongPtr(hListView,GWL_WNDPROC);
#pragma warning(disable: 4244)
	SetWindowLongPtr(hListView,GWL_USERDATA,LongPtr);
	SetWindowLongPtr(hListView,GWL_WNDPROC,reinterpret_cast<LONG_PTR>(ListViewExProc));
#pragma warning(default: 4244)

	// ^O쐬
	LV_COLUMN stLVCol;
	memset(&stLVCol,0,sizeof(stLVCol));
	stLVCol.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	stLVCol.fmt = LVCFMT_LEFT;
	
	stLVCol.cx = 60;
	stLVCol.iSubItem=FILTWND_TAB_SERVICEID;	
	stLVCol.pszText="T[rXID";					
	ListView_InsertColumn(hListView, stLVCol.iSubItem, &stLVCol);
	
	stLVCol.cx = 100;
	stLVCol.iSubItem=FILTWND_TAB_CHANNELNAME;
	stLVCol.pszText="`l";
	ListView_InsertColumn(hListView, stLVCol.iSubItem, &stLVCol);
	
	stLVCol.cx = 200;
	stLVCol.iSubItem=FILTWND_TAB_CURRENTEVENT;
	stLVCol.pszText="݂̔ԑg";
	ListView_InsertColumn(hListView, stLVCol.iSubItem, &stLVCol);

	stLVCol.cx = 50;
	stLVCol.iSubItem=FILTWND_TAB_PMTPID;
	stLVCol.pszText="PMT";
	ListView_InsertColumn(hListView, stLVCol.iSubItem, &stLVCol);

	stLVCol.cx = 200;
	stLVCol.iSubItem=FILTWND_TAB_PIDLIST;
	stLVCol.pszText="\PID";
	ListView_InsertColumn(hListView, stLVCol.iSubItem, &stLVCol);

#ifdef FILTWND_ENABLE_TIPWND
	m_hTip=CreateWindowEx(0,TOOLTIPS_CLASS,NULL,TTS_ALWAYSTIP|TTS_BALLOON,0,0,0,0,hListView,NULL,m_hInst,NULL);
	SendMessage(m_hTip,TTM_SETDELAYTIME,TTDT_AUTOPOP,MAKELPARAM(30000,0));	// 30sec\
	SendMessage(m_hTip,TTM_SETMAXTIPWIDTH,0,(LPARAM)FILTWND_TIP_WIDTH);
	SendMessage(m_hTip,TTM_SETTIPTEXTCOLOR,FILTWND_TIP_TEXTCOLOR,0);
	SendMessage(m_hTip,TTM_SETTIPBKCOLOR,FILTWND_TIP_BGCOLOR,0);
#endif

	m_hMenu = LoadMenu(m_hInst,MAKEINTRESOURCE(IDR_FILTWND_MENU));
}

// EBhETCYύX
void CFilterWnd::OnSize()
{
	CLock lock(&m_stLock);
	// EBhETCYύXقACeǏ]
	RECT rcRect,rcItem;
	HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
	GetWindowRect(m_hWnd,&rcRect);
	GetWindowRect(hListView,&rcItem);
	int iMarginX=rcItem.left-rcRect.left;
	int iMarginY=rcItem.top-rcRect.top;					
	SetWindowPos(hListView,NULL,0,0,rcRect.right-rcRect.left-iMarginX*2,rcRect.bottom-rcRect.top-iMarginY-iMarginX,SWP_NOMOVE|SWP_NOZORDER);
#ifdef FILTWND_ENABLE_TIPWND
	UpdateToolTip(hListView);
#endif
}

// L/؂ւ
void CFilterWnd::SetEnable(BOOL bEnable)
{			
	EnableWindow(GetDlgItem(m_hWnd,IDC_FILTER),bEnable);
	EnableWindow(GetDlgItem(m_hWnd,IDC_SERVICE_LIST),bEnable);
}

// Sf[^̏
void CFilterWnd::DeleteAllItems()
{
	CLock lock(&m_stLock);
	Reset();
	while(m_Service.size()>0)
	{
		FILTWND_SERVICEDATA *pItem=m_Service.begin()->second;
		pItem->pPIDs->clear();
		delete pItem->pPIDs;
		delete pItem;
		m_Service.erase(m_Service.begin()->first);
	}
	m_PosToService.clear();
	// ^C~OɂăfbhbNĂ܂̂ŁAWindowProcŏ
	PostMessage(m_hWnd,WM_FILTWND_DELETEALL,0,0);
}

// vOT[rXǉ
void CFilterWnd::AddService(WORD wServiceID,WORD wPmtPID)
{
	CLock lock(&m_stLock);
	LV_ITEM	*pstLI=new LV_ITEM;
	if(pstLI)
	{
		ZeroMemory(pstLI,sizeof(LV_ITEM));
		pstLI->pszText=new char[30];
		if(pstLI->pszText)
		{
			wsprintf(pstLI->pszText,"%d",wServiceID);

			FiltWndServiceListMap::iterator it=m_Service.find(wServiceID);
			if(it != m_Service.end())
			{
				// XV
				pstLI->mask=LVIF_TEXT;
				pstLI->iItem = it->second->wPosition;
				// ^C~OɂăfbhbNĂ܂̂ŁAWindowProcŏ
				PostMessage(m_hWnd,WM_FILTWND_ADDITEM,MAKEWPARAM(0,wPmtPID),(LPARAM)pstLI);
			} else {
				// VK
				FILTWND_SERVICEDATA *pItem = new FILTWND_SERVICEDATA;
				if(pItem)
				{
					ZeroMemory(pItem,sizeof(FILTWND_SERVICEDATA));
					pItem->wServiceID = wServiceID;
					pItem->wPosition = static_cast<WORD>(m_Service.size());
					pItem->pPIDs = new std::vector<WORD>;

					m_Service.insert(FiltWndServiceListPair(wServiceID,pItem));
					m_PosToService.insert(FiltWndPosToServiceIDPair(pItem->wPosition,wServiceID));
					
					pstLI->mask=LVIF_PARAM|LVIF_TEXT;
					pstLI->lParam=(LPARAM)pItem;
					pstLI->iItem = pItem->wPosition;
					// ^C~OɂăfbhbNĂ܂̂ŁAWindowProcŏ
					PostMessage(m_hWnd,WM_FILTWND_ADDITEM,MAKEWPARAM(wServiceID,wPmtPID),(LPARAM)pstLI);
				}
			}
		}
	}
}

// TuACeZbg
BOOL CFilterWnd::SetSubItem(WORD wServiceID,WORD wSubIndex,char *lpszText)
{
	CLock lock(&m_stLock);
	FiltWndServiceListMap::iterator it=m_Service.find(wServiceID);
	if(it!=m_Service.end())
	{
		HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
		LV_ITEM	stLI;
		memset(&stLI,0,sizeof(stLI));
		stLI.mask=LVIF_TEXT;
		stLI.iItem=it->second->wPosition;
		stLI.iSubItem=wSubIndex;
		stLI.pszText=lpszText;
		ListView_SetItem(hListView,&stLI);
#ifdef FILTWND_ENABLE_TIPWND
		switch(wSubIndex)
		{
		case FILTWND_TAB_CHANNELNAME:
			{
				lstrcpy(it->second->stTipString.szChName,lpszText);
			}
			break;
		}
#endif
		return TRUE;
	}
	return FALSE;
}

// PMTACeZbg
void CFilterWnd::SetPMT(WORD wServiceID,struct FILTWND_PMTITEM *pstPmtItem,int iNum)
{
	CLock lock(&m_stLock);
	int i,pos;
	char *pszBuf=new char[200];
	if(pszBuf)
	{
		struct FILTWND_SERVICEDATA *pItem=NULL;
		FiltWndServiceListMap::iterator it=m_Service.find(wServiceID);
		if(it!=m_Service.end())
		{
			pItem = it->second;
		}
		pszBuf[0]='\0';
		for(i=pos=0;i<iNum;i++)
		{
			// \pǉ
			char *p="???";
			int t=pstPmtItem[i].byType;
			switch(t)
			{
			case 0x01: p="mpg"; break;
			case 0x02: p="m2v"; break;
			case 0x06: p="m2s"; break;
			case 0x0d: p="dat"; break;
			case 0x0f: p="aac"; break;
			case 0x10: p="mp4"; break;
			case 0x1b: p="264"; break;	// <-
			}				
			pos+=wsprintf(pszBuf+pos,"0x%04X(%3s) ",pstPmtItem[i].wPID,p);
			if(pos+10>=200)
			{
				lstrcat(pszBuf,"...");
				break;
			}
			if(pItem)
			{
				pItem->pPIDs->push_back(pstPmtItem[i].wPID);
			}
		}
		PostMessage(m_hWnd,WM_FILTWND_SETSUBITEM,MAKEWPARAM(FILTWND_TAB_PIDLIST,wServiceID),(LPARAM)pszBuf);
	}
}

// T[rXZbg
void CFilterWnd::SetServiceName(WORD wServiceID,char *pszChName)
{
	CLock lock(&m_stLock);
	char *pszBuf=new char[100];
	if(pszBuf)
	{
		lstrcpy(pszBuf,pszChName);
		PostMessage(m_hWnd,WM_FILTWND_SETSUBITEM,MAKEWPARAM(FILTWND_TAB_CHANNELNAME,wServiceID),(LPARAM)pszBuf);
	}
}

#define EVS_STARTDATE		(0x00000001)
#define EVS_ENDDATE			(0x00000002)
#define EVS_STARTTIME		(0x00000004)
#define EVS_STARTOENDTIME	(0x0000000C)
#define EVS_TITLE			(0x00000010)
#define EVS_DETAIL			(0x00000020)
// Tip쐬p
#define EVS_MAKETIPSTR		(0x80000000)
#define EVS_TITLE_EXT		(0x40000000)


void CFilterWnd::MakeEventStrings(DWORD dwFlags,char *lpszBuf,int iBufLen,WORD wStartMJD,DWORD dwStartSec,WORD wEndMJD,DWORD dwEndSec,char *szTitle,char *szDetail)
{	
	int pos=0,size;
	if(dwFlags&EVS_STARTDATE)
	{
		long y,m,d;
		CTsStream::Convert_mjd_to_date(wStartMJD,&y,&m,&d);
		size=wsprintf(lpszBuf+pos,"%04d/%02d/%02d ",y,m,d);
		pos+=size;
		iBufLen-=size;
	}
	if(dwFlags&EVS_STARTTIME)
	{
		long sh,sm,ss;
		CTsStream::Convert_time(dwStartSec,&sh,&sm,&ss);
		size=wsprintf(lpszBuf+pos,"%02d:%02d",sh,sm);
		pos+=size;
		iBufLen-=size;
	}
	if(dwFlags&EVS_STARTOENDTIME)
	{
		if(dwEndSec!=0xFFFFFFFF)
		{
			size=wsprintf(lpszBuf+pos," - ");
			pos+=size;
			iBufLen-=size;
			if(dwFlags&EVS_ENDDATE)
			{
				long y,m,d;
				CTsStream::Convert_mjd_to_date(wEndMJD,&y,&m,&d);
				size=wsprintf(lpszBuf+pos,"%04d/%02d/%02d ",y,m,d);
				pos+=size;
				iBufLen-=size;
			}
			long eh,em,es;
			CTsStream::Convert_time(dwEndSec,&eh,&em,&es);
			size=wsprintf(lpszBuf+pos,"%02d:%02d",eh,em);		
		} else {
			size=wsprintf(lpszBuf+pos," ");		
		}
		pos+=size;
		iBufLen-=size;
	}
	if(dwFlags&EVS_TITLE)
	{
		if(dwFlags&EVS_MAKETIPSTR)
		{
			if(dwFlags&EVS_TITLE_EXT)
			{
				size=wsprintf(lpszBuf+pos," << %s >>",szTitle);
			} else {
				size=wsprintf(lpszBuf+pos," %s",szTitle);
			}
		} else {
			if(dwFlags&EVS_TITLE_EXT)
			{
				size=wsprintf(lpszBuf+pos," << %s >>",szTitle);
			} else {
				size=wsprintf(lpszBuf+pos," %s",szTitle);
			}
		}
		pos+=size;
		iBufLen-=size;
	}
	if(dwFlags&EVS_DETAIL)
	{
		if(dwFlags&EVS_MAKETIPSTR)
		{
			if(dwFlags&EVS_TITLE)
			{
				size=wsprintf(lpszBuf+pos,"\n%s",szDetail);
			} else {
				size=wsprintf(lpszBuf+pos," %s",szDetail);
			}
		} else {
			size=wsprintf(lpszBuf+pos," %s",szDetail);
		}
		pos+=size;
		iBufLen-=size;
	}
}

// ݂̔ԑgZbg(f[^͌Ăяo֖₢킹)
void CFilterWnd::SetCurrentEPG(WORD wDate,DWORD dwSec,PF_FILTERWND_EPGCALLBACK *lpFunc,LPVOID lpContext)
{
	CLock lock(&m_stLock);
	if(!lpFunc) return;
	/*
	// Iterator
	FiltWndServiceListMap::iterator it;
	for(it=m_Service.begin();it!=m_Service.end();it++)
	{
		FILTWND_SERVICEDATA *pstSericeData = it->second;
		//
	*/

	// ListItem
	int i,n;
	HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
	n=ListView_GetItemCount(hListView);
	for(i=0;i<n;i++)
	{
		FILTWND_SERVICEDATA *pstSericeData=NULL;
		LV_ITEM	stLI;
		stLI.mask=LVIF_PARAM;
		stLI.iItem=i;
		ListView_GetItem(hListView,&stLI);
		pstSericeData=(FILTWND_SERVICEDATA*)stLI.lParam;
		//
		if(pstSericeData)
		{
			BOOL bFindCurrent=FALSE;
			struct TSEPG_PROGRAM stProg;
			if(lpFunc(pstSericeData->wServiceID,wDate,dwSec,&stProg,TRUE,lpContext))
			{
				bFindCurrent=TRUE;
				BOOL bUpdate=FALSE;
				if(stProg.byType != pstSericeData->stCurrnetEvent.stProgram.byType) bUpdate = TRUE;
				if(stProg.wEventID != pstSericeData->stCurrnetEvent.stProgram.wEventID) bUpdate = TRUE;
				if(stProg.byVersion != pstSericeData->stCurrnetEvent.stProgram.byVersion) bUpdate = TRUE;
				if(bUpdate)
				{
					char *pszBuf=new char[100];
					if(pszBuf)
					{
						pstSericeData->stCurrnetEvent.stProgram = stProg;

						MakeEventStrings(EVS_STARTOENDTIME|EVS_TITLE,pszBuf,100,0,stProg.dwStartTime,0,stProg.dwEndTime,stProg.szTitle,stProg.szText);
						PostMessage(m_hWnd,WM_FILTWND_SETSUBITEM,MAKEWPARAM(FILTWND_TAB_CURRENTEVENT,pstSericeData->wServiceID),(LPARAM)pszBuf);
						
#ifdef FILTWND_ENABLE_TIPWND
						// Tip\p
						MakeEventStrings(EVS_STARTDATE|EVS_STARTOENDTIME|EVS_TITLE|EVS_DETAIL|EVS_MAKETIPSTR,
							pstSericeData->stTipString.szPresentEvent,FILTWND_TIP_MAX_EVENTSTR,
							stProg.wStartMJD,stProg.dwStartTime,stProg.wEndMJD,stProg.dwEndTime,
							stProg.szTitle,stProg.szText);
#endif
					}
				}
#ifdef FILTWND_ENABLE_TIPWND
			} else {
				char *pszBuf=new char[1];
				if(pszBuf)
				{
					pszBuf[0]='\0';
					pstSericeData->stCurrnetEvent.stProgram.wStartMJD=0;
#ifdef FILTWND_ENABLE_TIPWND
					pstSericeData->stTipString.szPresentEvent[0]='\0';
#endif
					PostMessage(m_hWnd,WM_FILTWND_SETSUBITEM,MAKEWPARAM(FILTWND_TAB_CURRENTEVENT,pstSericeData->wServiceID),(LPARAM)pszBuf);
				}
			}
			// ݂̎̃Cxg
			if(bFindCurrent)
			{
#ifdef FILTWND_TIP_NEXTEVENT
				struct TSEPG_PROGRAM stNext;
				WORD wDateNext;
				DWORD dwSecNext;
				if(stProg.dwEndTime+1<24*60*60)
				{
					wDateNext=stProg.wEndMJD;
					dwSecNext=stProg.dwEndTime+1;
				} else {
					wDateNext=stProg.wEndMJD+1;
					dwSecNext=0;
				}
				if(lpFunc(pstSericeData->wServiceID,wDateNext,dwSecNext,&stNext,FALSE,lpContext))
				{
					BOOL bUpdate=FALSE;
					if(stNext.byType != pstSericeData->stNextEvent.stProgram.byType) bUpdate = TRUE;
					if(stNext.wEventID != pstSericeData->stNextEvent.stProgram.wEventID) bUpdate = TRUE;
					if(stNext.byVersion != pstSericeData->stNextEvent.stProgram.byVersion) bUpdate = TRUE;
					if(bUpdate)
					{
						pstSericeData->stNextEvent.stProgram = stNext;
						// Tip\p
						MakeEventStrings(EVS_STARTDATE|EVS_STARTOENDTIME|EVS_TITLE|EVS_DETAIL|EVS_MAKETIPSTR,
							pstSericeData->stTipString.szFutureEvent,FILTWND_TIP_MAX_EVENTSTR,
							stNext.wStartMJD,stNext.dwStartTime,stNext.wEndMJD,stNext.dwEndTime,
							stNext.szTitle,stNext.szText);
					}
				} else {
					pstSericeData->stTipString.szFutureEvent[0]='\0';
				}
#endif
#endif
			} else {
#ifdef FILTWND_ENABLE_TIPWND
				pstSericeData->stNextEvent.stProgram.wStartMJD=0;
				pstSericeData->stTipString.szFutureEvent[0]='\0';
#endif
			}
		}
	}
}

// tB^f[^
void CFilterWnd::CreateFilterList(WORD** ppwServiceList,int *piServiceNum,WORD **ppwPidList,int *piPidNum)
{
	CLock lock(&m_stLock);

	HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
	std::set<WORD> Allow,Deny,DenyService,Filter;
	std::set<WORD>::iterator itFilt;
	unsigned int i,j,n,nPID;	

	n=ListView_GetItemCount(hListView);
	for(i=0;i<n;i++)
	{
		FILTWND_SERVICEDATA *pstSericeData=NULL;
		LV_ITEM	stLI;
		stLI.mask=LVIF_PARAM;
		stLI.iItem=i;
		ListView_GetItem(hListView,&stLI);
		pstSericeData=(FILTWND_SERVICEDATA*)stLI.lParam;
		//		
		if(pstSericeData)
		{
			BOOL bState=ListView_GetCheckState(hListView,stLI.iItem);
			if(bState)
			{
				// L	
				nPID=static_cast<WORD>(pstSericeData->pPIDs->size());
				for(j=0;j<nPID;j++)
				{
					Allow.insert(pstSericeData->pPIDs->at(j));
				}
			} else {
				// 				
				DenyService.insert(pstSericeData->wServiceID);
				nPID=static_cast<WORD>(pstSericeData->pPIDs->size());
				for(j=0;j<nPID;j++)
				{
					Deny.insert(pstSericeData->pPIDs->at(j));
				}
			}
		}
	}
	// ɂāALɂȂ̂
	Filter.clear();
	for(itFilt=Deny.begin();itFilt!=Deny.end();itFilt++)
	{
		if(Allow.find(*itFilt)!=Allow.end()) continue;
		Filter.insert(*itFilt);
	}
	// T[rXXg쐬
	if(ppwServiceList && piServiceNum && DenyService.size() > 0)
	{
		*piServiceNum = static_cast<int>(DenyService.size());
		*ppwServiceList = new WORD[*piServiceNum];
		for(itFilt=DenyService.begin(),i=0;itFilt!=DenyService.end();itFilt++)
		{
			(*ppwServiceList)[i++]=*itFilt;
		}
	}
	// PIDXg쐬
	if(ppwPidList && piPidNum && Filter.size() > 0)
	{
		*piPidNum = static_cast<int>(Filter.size());
		*ppwPidList = new WORD[*piPidNum];
		for(itFilt=Filter.begin(),i=0;itFilt!=Filter.end();itFilt++)
		{
			(*ppwPidList)[i++]=*itFilt;
		}
	}
}

// tB^
void CFilterWnd::OnPacketFilterBtn(BOOL bForceOff)
{
	if(m_bPacketFilter||bForceOff)
	{
		// tB^폜
		m_bPacketFilter=FALSE;
		CheckDlgButton(m_hWnd,IDC_FILTER,BST_UNCHECKED);
		
		if(m_lpSetFilterFunc) m_lpSetFilterFunc(NULL,0,NULL,0,m_lpSetFilterContext);
		
		EnableWindow(GetDlgItem(m_hWnd,IDC_SERVICE_LIST),TRUE);
	} else {
		// tB^ǉ
		m_bPacketFilter=TRUE;
		WORD *pDenyServiceList=NULL,*pDenyPidList=NULL;
		int iDenyServiceNum=0,iDenyPidNum=0;
		CheckDlgButton(m_hWnd,IDC_FILTER,BST_CHECKED);
		CreateFilterList(&pDenyServiceList,&iDenyServiceNum,&pDenyPidList,&iDenyPidNum);
		
		if(m_lpSetFilterFunc) m_lpSetFilterFunc(pDenyServiceList,iDenyServiceNum,pDenyPidList,iDenyPidNum,m_lpSetFilterContext);
		
		if(pDenyServiceList) delete []pDenyServiceList;
		if(pDenyPidList) delete []pDenyPidList;
		EnableWindow(GetDlgItem(m_hWnd,IDC_SERVICE_LIST),FALSE);
	}
}

#ifdef FILTWND_ENABLE_TIPWND
// XN[}EXW}EX̃Xgr[ACe(Report)擾
// piSubItem==NULL̎́AACeԍ̂݌
BOOL CFilterWnd::MousePt_to_ListViewItem(POINT ptMouse,int *piItem,int *piSubItem)
{
	HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
	if((GetWindowLong(hListView,GWL_STYLE)&LVS_TYPEMASK) != LVS_REPORT) return FALSE;

	// \ACe
	int iItemTop=ListView_GetTopIndex(hListView);
	int iItemInPageCount=ListView_GetCountPerPage(hListView);
	int iItemBottom=iItemTop+iItemInPageCount;

	int iItemCount=ListView_GetItemCount(hListView);
	if(iItemCount==0) return FALSE;

	// ԏ̍
	RECT rcItem;
	if(!ListView_GetItemRect(hListView,iItemTop,&rcItem,LVIR_BOUNDS)) return FALSE;
	int iItemHeight=rcItem.bottom-rcItem.top;
	if(iItemHeight<=0) return FALSE;

	// }EXWListViewWɕϊAԏ̍ڂ̍(0,0)Ƃ΍Wɕϊ
	ScreenToClient(hListView,&ptMouse);
	ptMouse.x-=rcItem.left;
	ptMouse.y-=rcItem.top;

	// ݂̑IACevZA͈͊OȂG[
	int iSelecting = iItemTop + ptMouse.y / iItemHeight;
	if(iSelecting<0 || iSelecting>=ListView_GetItemCount(hListView)) return FALSE;
	if(piItem) *piItem=iSelecting;

	// TuACe擾
	if(piSubItem)
	{
		int iColumn,iColumnNum;
		HWND hHeader=ListView_GetHeader(hListView);
		if(!hHeader) return FALSE;
		iColumnNum=Header_GetItemCount(hHeader);
		for(iColumn=0;iColumn<iColumnNum;iColumn++)
		{
			if(!Header_GetItemRect(hHeader,iColumn,&rcItem)) return FALSE;
			if(rcItem.left<=ptMouse.x && ptMouse.x <= rcItem.right)
			{
				*piSubItem=iColumn;
				break;
			}
		}
	}
	return TRUE;
}
#endif

#ifdef FILTWND_ENABLE_TIPWND
// c[Tipp̍XV
void CFilterWnd::UpdateToolTip(HWND hListView)
{
	CLock lock(&m_stLock);

	unsigned int iItemTop=ListView_GetTopIndex(hListView);
	unsigned int iItemInPageCount=ListView_GetCountPerPage(hListView);
	unsigned int iItemBottom=iItemTop+iItemInPageCount;

	unsigned int iItemCount=ListView_GetItemCount(hListView);
	if(iItemCount==0) return;

	// ő\ACeωA̕ǉ
	while(m_Tooltip.size()<iItemInPageCount)
	{
		FILTWND_TOOLTIP stNew;
		ZeroMemory(&stNew,sizeof(FILTWND_TOOLTIP));
		stNew.stToolTipInfo.cbSize=sizeof(FILTWND_TOOLTIP);
		stNew.stToolTipInfo.hwnd=hListView;
		stNew.stToolTipInfo.uFlags=TTF_SUBCLASS;
		stNew.stToolTipInfo.uId=m_Tooltip.size()+1;		
		stNew.stToolTipInfo.lpszText="";
		if(ListView_GetItemRect(hListView,0,&stNew.stToolTipInfo.rect,LVIR_BOUNDS))
		{
			unsigned int iHeight=stNew.stToolTipInfo.rect.bottom-stNew.stToolTipInfo.rect.top;
			unsigned int iTop = stNew.stToolTipInfo.rect.top;
			stNew.stToolTipInfo.rect.top    = stNew.stToolTipInfo.rect.top + iHeight * (stNew.stToolTipInfo.uId-1);
			stNew.stToolTipInfo.rect.bottom = stNew.stToolTipInfo.rect.top + iHeight;
			SendMessage(m_hTip,TTM_ADDTOOL,0,(LPARAM)&stNew.stToolTipInfo);			
		}
		m_Tooltip.push_back(stNew);
	}
	// ACeXV
	RECT rcItem;
	if(ListView_GetItemRect(hListView,0,&rcItem,LVIR_BOUNDS))
	{
		unsigned int i,n;
		n=static_cast<unsigned int>(m_Tooltip.size());		
		for(i=0;i<n;i++)
		{			
			TOOLINFO *pToolInfo=&m_Tooltip[i].stToolTipInfo;
			// XV
			pToolInfo->rect.left=rcItem.left;
			pToolInfo->rect.right=rcItem.right;
			SendMessage(m_hTip,TTM_NEWTOOLRECT,0,(LPARAM)pToolInfo);
			// eLXgXV
			char buf[FILTWND_TIP_MAX_EVENTSTR*2]=FILTWND_TIP_NODATA;
			pToolInfo->lpszText=buf;
			if(iItemTop+i<iItemCount)
			{
				FiltWndPosToServiceIDMap::iterator itPos=m_PosToService.find(iItemTop+i);
				if(itPos!=m_PosToService.end())
				{
					FiltWndServiceListMap::iterator itService=m_Service.find(itPos->second);
					if(itService!=m_Service.end())
					{
						FILTWND_SERVICEDATA *pItem=itService->second;
						int pos=0;
						pos+=wsprintf(pToolInfo->lpszText+pos,"%s(%d)",pItem->stTipString.szChName,pItem->wServiceID);
						if(pItem->stTipString.szPresentEvent[0])
						{
							pos+=wsprintf(pToolInfo->lpszText+pos,"\n\n݂̔ԑg>>\n%s",pItem->stTipString.szPresentEvent);
#ifdef FILTWND_TIP_NEXTEVENT
							if(pItem->stTipString.szFutureEvent[0])
							{
								pos+=wsprintf(pToolInfo->lpszText+pos,"\n\n̔ԑg>>\n%s",pItem->stTipString.szFutureEvent);
							}
#endif
						}
					}
				}				
			}
			SendMessage(m_hTip,TTM_UPDATETIPTEXT,0,(LPARAM)pToolInfo);
		}
	}
}
#endif

// ݃NbNĂ擪ACȅ
BOOL CFilterWnd::OnRecSchedule(int iFlg)
{
	CLock lock(&m_stLock);
	HWND hListView=GetDlgItem(m_hWnd,IDC_SERVICE_LIST);
	FILTWND_SERVICEDATA *pstSericeData=NULL;

	int pos=ListView_GetNextItem(GetDlgItem(m_hWnd,IDC_SERVICE_LIST),-1,LVNI_ALL|LVNI_SELECTED);
	if(pos==-1) return FALSE;

	LV_ITEM	stLI;
	stLI.mask=LVIF_PARAM;
	stLI.iItem=pos;
	ListView_GetItem(hListView,&stLI);
	pstSericeData=(FILTWND_SERVICEDATA*)stLI.lParam;
	if(!pstSericeData) return FALSE;
	FILTWND_EVENTDATA *pData=NULL;
	switch(iFlg)
	{
	case 0:
		pData=&pstSericeData->stCurrnetEvent;
		break;
	case 1:
		pData=&pstSericeData->stNextEvent;
		break;
	}
	if(!pData) return FALSE;
	if(pData->stProgram.wStartMJD==0) return FALSE;

	if(!m_lpScheduleTimeFunc) return FALSE;
	return m_lpScheduleTimeFunc(pstSericeData->wServiceID,pData->stProgram.wStartMJD,pData->stProgram.dwStartTime,pData->stProgram.dwDuration,pData->stProgram.szTitle,m_lpScheduleTimeContext);
}