#pragma once

#include "../Common/Buffer.h"
#include "../lib/IB25Decoder.h"
#include "TsHeader.h"
#include "TsStreamCallback.h"
#include "TsPsiSection.h"
#include "TsEpgMgr.h"
#include <map>
#include <set>

#define CONTINUITYCOUNTER_INIT	0xFF
#define TABLEID_INIT			0xFFFFFFFF

#define TS_OUTPUT_DEFAULT		0	// ftHgTsXg[o͔ԍ

struct TS_StreamInfo
{
	double	dSpeed;

	WORD	wTsPacketLen;
	DWORD	dwTsSyncFail;
	DWORD	dwTsPacketDrop;
	DWORD	dwScramblePacket;

	struct {
		WORD wDate;
		DWORD dwTime;
		//
		DWORD dwGetTick;	// tA擾̃VXe
	} stCurrentTime;

	DWORD	dwRecieveBufferOverflow;
	DWORD	dwSyncBufferOverflow;
	DWORD	dwStreamBufferOverflow;
};

struct TS_PidInfo
{
	WORD type;						// PID̃^Cv
	BYTE old_continuity_counter;	// Ocontinuity_counter

	DWORD dwCurrentID;
	BOOL bStartFlg;					// JnĂ邩̃tO(ŏunit_start܂ł̓f[^Ȃ)
	CBuffer	*pBuf;					// PSIp̈ꎞf[^ۑpobt@
};

typedef std::set<WORD> TsFilterSet;
struct TS_Filter
{
	BOOL bRejectPATFlg;		// TRUEȂŵ̂ubNAFALSEȂŵ̈ȊOubN
	TsFilterSet PATFilter;	// w肷T[rXID(ubNPATUɂ肩)
	BOOL bRejectPIDFlg;
	TsFilterSet PIDFilter;	// w肷PID(ubNƃf[^͔Ȃ)
};
typedef std::map<WORD,struct TS_Filter> TsFilterMap;
typedef std::pair<WORD,struct TS_Filter> TsFilterPair;

class CTsStream
{
public:
	CTsStream(IB25Decoder *lpB25Decorder=NULL);
	~CTsStream(void);
	void ResetStream(BOOL bNewAlloc=TRUE);
	
	void SetDecodePSI(BOOL bDecode);
	void SetEpgProc(BOOL bProc);
	void SetCallbackFunc(PF_TSENGINE_CALLBACK lpFunc,LPVOID lpContext);

	BOOL AddOutput(WORD wOutNo);
	BOOL DelOutput(WORD wOutNo);
	void ClearAllFilter(WORD wOutNo,BOOL bAllOutput=FALSE);
	void SetFilter(WORD wOutNo,BOOL bRejectPATFlg,WORD *pwServiceList,int iServiceNum,BOOL bRejectPIDFlg,WORD *pwPidList,int iPidNum);

	TsFilterMap m_Filter;

	void OnTsStream(BYTE *pStream,DWORD dwSize,TS_StreamInfo *lpstTsStreamInfo);
	void OnTsStreamTerminate(TS_StreamInfo *lpstTsStreamInfo);

	// EpgMgr
	struct TSEPG_PROGRAM* FindEpgDataByTime(WORD wServiceID,WORD wDateMJD,DWORD dwTime,BOOL bDeleteMode=FALSE)
	{
		return m_EpgMgr.FindEpgDataByTime(wServiceID,wDateMJD,dwTime,bDeleteMode);
	}
	void FreeEpgDataMemory(struct TSEPG_PROGRAM *pData,int iCount)
	{
		m_EpgMgr.FreeEpgDataMemory(pData,iCount);
	}

	// f[^ϊ
	static void Convert_jpn_arib_char(char *str,size_t len,char *ret_buf,size_t ret_len);
	static void Convert_mjd_to_date(unsigned short mjd,long *p_year,long *p_month,long *p_day);
	static WORD Convert_date_to_mjd(long year,long month,long day);
	static DWORD Convert_bcd_to_time(unsigned char bcd_hour,unsigned char bcd_min,unsigned char bcd_sec);
	static void Convert_time(DWORD time,long *p_hour,long *p_min,long *p_sec);
	static DWORD Convert_bcd_to_dword(DWORD dwData);
protected:
	CBuffer *m_pBuf;
	CBuffer *m_pAlignment;
	BOOL m_bDecodePSI;
	BOOL m_bProcEpg;
	IB25Decoder *m_pB25Decorder;
	PF_TSENGINE_CALLBACK *m_pCallbackFunc;
	LPVOID m_pCallbackContext;
	BYTE *m_pPayloadTemp;
	struct TS_PidInfo m_stPidTable[TS_PID_MAX];

	void DoCallback(UINT uType=TSENG_CALLBACK_NONE,WORD wStreamID=0,void *pData=NULL,DWORD dwVal=0)
	{
		if(m_pCallbackFunc)	m_pCallbackFunc(uType,wStreamID,(BYTE*)pData,dwVal,m_pCallbackContext);
	}

	void TsStreamStreamProc(BYTE *pAlignmentStream,DWORD dwAlignmentStreamLen,TS_StreamInfo *lpstTsStreamInfo);
	void MakeFakePAT(struct TS_Filter &stFilter,struct TS_header &stTsHeader,BYTE *pBuf,int pos,int len);

// PSIZNV֘A
	BOOL OnSection(struct TS_header &stTsHeader,struct TS_section_header &stSecHeader,BYTE *pData,size_t len,PF_SECTION_CALLBACK *lpCallback,LPVOID lpContext);
	int CheckSection(struct TS_header &stTsHeader,struct TS_section_header &stSecHeader,CBuffer *pBuffer,PF_SECTION_CALLBACK *lpCallback,LPVOID lpContext);

	static BOOL CALLBACK OnPAT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnPMT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnNIT_SIT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnTOT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnSDT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);
	static BOOL CALLBACK OnEIT(struct TS_header &stTsHeader,struct TS_section_header&stSecHeader,CBuffer *pBuf,LPVOID lpContext);

	int OnTable(struct TS_header &stTsHeader,struct TS_section_header &stSecHeader,BYTE *pData,size_t &len,CBuffer **ppBuf);			

	// EIT XV`FbNpL[̎擾
	// xxxxxxxx-yy-zz x=StreamID y=TableID z=SectionID
	inline DWORD GetEitKey(WORD wStreamID,BYTE byTableID,BYTE bySectionID)
	{
		DWORD ret;
		ret = ((DWORD)wStreamID)<<16 | ((DWORD)byTableID)<<8 | bySectionID;
		return ret;
	};	

	// ݂̃Xg[
	struct TS_CurrentStreamInfo
	{
		// TOT
		WORD wMJDdate;
		DWORD dwBCDtime;

		// PAT
		WORD wStreamID;
		BYTE byPatVerNo;

		// NIT/SIT
		BOOL bNitRecv;
		BYTE byNitVerNo;

		// SDT
		BOOL bSdtRecv;
		BYTE bySdtVerNo;

		// EIT p/f Actual
		BOOL bEitPfActualRecv;
		BYTE byEitPfActualVerNo;
	} m_stCurrentStream;	

	// PAT
	struct TS_PAT m_CurrentPAT;

	// PMT
	TS_StreamPmt m_CurrentPmt;

	std::map<DWORD,WORD> m_stCurrentStreamEitVerNo;

	// EPGf[^x[X
	CTsEPGMgr m_EpgMgr;
};
